unit ConfigureFrm;

interface

uses
  Windows, SysUtils, Classes, Forms, MainFrm, MPlayer, registry,
  jpeg, MMSystem, ComCtrls, StdCtrls, ExtCtrls, Controls;

type
  TConfigureForm = class(TForm)
    btnOk: TButton;
    btnCancel: TButton;
    VolumeTrackBar: TTrackBar;
    MediaPlayer: TMediaPlayer;
    Image: TImage;
    Label1: TLabel;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure VolumeTrackBarChange(Sender: TObject);
    procedure MediaPlayerNotify(Sender: TObject);
  public
    { Public declarations }
  private
    MP3FileName: String;
    procedure LoadMP3;
    procedure SetMPVolume(MP: TMediaPlayer; Volume: Integer);
    function GetTempDirectory: String;
  end;

var
  ConfigureForm: TConfigureForm;

implementation

uses ScreenSaverUtils;

{$R *.DFM}

procedure TConfigureForm.FormCreate(Sender: TObject);
var
    Registry: TRegistry;
begin
    Caption := 'Configure ' + Application.Title;

    Registry := TRegistry.Create();
    Registry.RootKey := HKEY_CURRENT_USER;
    Registry.OpenKey('Software\www.StaggeringStories.net\Throwing Shapes with Cre''at', True);
    try
        VolumeTrackBar.Position := Registry.ReadInteger('AudioVolume');
    except
        VolumeTrackBar.Position := 1000;
    end;

    MP3FileName := GetTempDirectory + 'throwingshapes.mp3';
    LoadMP3();
    MediaPlayer.FileName := MP3FileName;
    MediaPlayer.Open;

    SetMPVolume(MediaPlayer, VolumeTrackBar.Position);
    MediaPlayer.Play;
    MediaPlayer.Notify := False;
end;


procedure TConfigureForm.btnOkClick(Sender: TObject);
var
  Registry: TRegistry;
begin
    Registry := TRegistry.Create();
    Registry.RootKey := HKEY_CURRENT_USER;
    Registry.OpenKey('Software\www.StaggeringStories.net\Throwing Shapes with Cre''at', True);
    Registry.WriteInteger('AudioVolume', VolumeTrackBar.Position);
    Close;
end;


procedure TConfigureForm.btnCancelClick(Sender: TObject);
begin
    Close;
end;


procedure TConfigureForm.VolumeTrackBarChange(Sender: TObject);
begin
     SetMPVolume(MediaPlayer, VolumeTrackBar.Position);
end;


procedure TConfigureForm.SetMPVolume(MP: TMediaPlayer; Volume: Integer);
var
    p: MCI_DGV_SETAUDIO_PARMS;
begin
    // Volume: 0 - 1000
    p.dwCallback := 0;
    p.dwItem := MCI_DGV_SETAUDIO_VOLUME;
    p.dwValue := Volume;
    p.dwOver := 0;
    p.lpstrAlgorithm := nil;
    p.lpstrQuality := nil;
    mciSendCommand(MP.DeviceID, MCI_SETAUDIO, MCI_DGV_SETAUDIO_VALUE or MCI_DGV_SETAUDIO_ITEM, Cardinal(@p));
end;


procedure TConfigureForm.LoadMP3;
var
    ResourceStream: TResourceStream;
    FileStream: TFileStream;
begin
    ResourceStream := TResourceStream.Create(hInstance, '#128', 'MP3');
    try
        FileStream := TFileStream.Create(MP3FileName, fmCreate);
        try
            FileStream.CopyFrom(ResourceStream, 0);
        finally
            FileStream.Free;
        end;
    finally
        ResourceStream.Free;
    end;
end;


function TConfigureForm.GetTempDirectory: String;
var
    tempFolder: array[0..MAX_PATH] of Char;
begin
    GetTempPath(MAX_PATH, @tempFolder);
    result := StrPas(tempFolder);
end;


procedure TConfigureForm.MediaPlayerNotify(Sender: TObject);
begin
     //MediaPlayer.Notify := False;
     //MediaPlayer.Stop;
     //MediaPlayer.Rewind;
     //MediaPlayer.Play;
     //MediaPlayer.Notify := True;
end;


procedure TConfigureForm.FormDestroy(Sender: TObject);
begin
  //FSettings.Free;
  if ParentSaverHandle <> 0 then
  begin
    // The parent of the handle passed to us is
    // the main Properties dialog box
    EnableWindow(GetParent(ParentSaverHandle), True);
    SetForegroundWindow(GetParent(ParentSaverHandle));
  end;
end;


end.
