unit MainFrm;

interface

{$R resources.RES}

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Jpeg, ExtCtrls, MPlayer, Order, MMSystem;

type
  TMainForm = class(TForm)
    Timer: TTimer;
    Panel: TPanel;
    Image: TImage;
    MediaPlayer: TMediaPlayer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    FLastMousePos: TPoint;
    Order: TOrder;
    MP3FileName: String;
    AudioVolume: Integer;
    procedure ResizeImage;
    procedure NextImage;
    procedure LoadImage;
    procedure LoadMP3;
    function GetTempDirectory: String;
    function LoadStringResource: TStrings;
    procedure SetMPVolume(MP: TMediaPlayer; Volume: Integer);
    //function GetMPVolume(MP: TMediaPlayer): Integer;
  end;

const
  MCI_SETAUDIO = $0873;
  MCI_DGV_SETAUDIO_VOLUME = $4002;
  MCI_DGV_SETAUDIO_ITEM = $00800000;
  MCI_DGV_SETAUDIO_VALUE = $01000000;
  MCI_DGV_STATUS_VOLUME = $4019;

type
  MCI_DGV_SETAUDIO_PARMS = record
    dwCallback: DWORD;
    dwItem: DWORD;
    dwValue: DWORD;
    dwOver: DWORD;
    lpstrAlgorithm: PChar;
    lpstrQuality: PChar;
  end;

type
  MCI_STATUS_PARMS = record
    dwCallback: DWORD;
    dwReturn: DWORD;
    dwItem: DWORD;
    dwTrack: DWORD;
  end;


var
  MainForm: TMainForm;

implementation

uses ScreenSaverUtils, Registry, SingleInstance;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
var
  Rect: TRect;
begin
    if ScreenSaverMode = ssRun then
    begin
        MainForm.Top := 0;
        MainForm.Left := 0;
        MainForm.Width := Screen.Width;
        MainForm.Height := Screen.Height;
        ShowCursor(False);
        FLastMousePos.x := -1;
        FLastMousePos.y := -1;
        MP3FileName := GetTempDirectory + 'throwingshapes.mp3';
    end
    else
    begin
        FreeMutex;
        GetWindowRect(ParentSaverHandle, Rect);

        MainForm.Top := Rect.Top;
        MainForm.Left := Rect.Left;
        MainForm.Width := Rect.Right - Rect.Left;
        MainForm.Height := Rect.Bottom - Rect.Top;
    end;

    Order := TOrder.Create(LoadStringResource);
end;


procedure TMainForm.FormDestroy(Sender: TObject);
var
    Dummy: Word;
begin
    if ScreenSaverMode = ssRun then
    begin
      SystemParametersInfo(SPI_SCREENSAVERRUNNING,0,@Dummy,0);
      ReleaseCapture;
      ShowCursor(True);
    end;
end;


procedure TMainForm.FormActivate(Sender: TObject);
var
    Registry: TRegistry;
begin
    Caption := 'Configure ' + Application.Title;

    if ScreenSaverMode = ssRun then
    begin
        Registry := TRegistry.Create();
        Registry.RootKey := HKEY_CURRENT_USER;
        Registry.OpenKey('Software\www.StaggeringStories.net\Throwing Shapes with Cre''at', True);

        try
            AudioVolume := Registry.ReadInteger('AudioVolume');
        except
            AudioVolume := 1000;
        end;

        if AudioVolume > 0 then
        begin
            LoadMP3();
            SetMPVolume(MediaPlayer, AudioVolume);
        end;
    end
    else
    begin
         AudioVolume := 0;
    end;

    ResizeImage;
    Application.ProcessMessages;

    if ScreenSaverMode = ssRun then
    begin
        MediaPlayer.Close;
        MediaPlayer.FileName := MP3FileName;
        MediaPlayer.Open;
    end;

    NextImage;
end;


procedure TMainForm.TimerTimer(Sender: TObject);
begin
    if ScreenSaverMode = ssPreview then
    begin
        if IsWindowVisible(ParentSaverHandle) = False then
        begin
            Close;
        end
        else
        begin
            NextImage();
        end;
    end
    else
    begin
        NextImage();
    end;
end;


procedure TMainForm.NextImage;
begin
    Timer.Enabled := False;
    LoadImage;
    Timer.Interval := Order.GetDisplayDuration;

    if (Order.IsStartOfList = True) and (AudioVolume > 0) then
    begin
         MediaPlayer.Stop;
         MediaPlayer.Rewind;
         MediaPlayer.Play;
    end;

    Order.NextImage;
    Timer.Enabled := True;
end;


procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
    Close;
end;


procedure TMainForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
begin
    if ScreenSaverMode = ssRun then
    begin
        if (FLastMousePos.X = -1) and (FLastMousePos.Y = -1) then
        begin
            FLastMousePos.X := X;
            FLastMousePos.Y := Y;
        end
        else
        begin
            if (Abs(X - FLastMousePos.X) > 5) and (Abs(Y - FLastMousePos.Y) > 5) then
            begin
                FLastMousePos.X := X;
                FLastMousePos.Y := Y;
                Close;
            end;
        end;
    end;
end;


procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
    if ScreenSaverMode = ssRun then
    begin
        Timer.Enabled := False;
        ShowCursor(True);
        CanClose := PromptIfPasswordNeeded(Handle);

        if not CanClose then
        begin
            ShowCursor(False);
            Timer.Enabled := True;
        end;
    end
    else
    begin
        CanClose := True;
    end;
end;


procedure TMainForm.ResizeImage;
var
   NewWidth, NewHeight: Integer;
begin
    NewWidth := Panel.ClientWidth;
    NewHeight := Round(Image.Picture.Height / (Image.Picture.Width / NewWidth));
    if NewHeight > Panel.ClientHeight then
    begin
        //We must work in both axes!
        NewHeight := Panel.ClientHeight;
        NewWidth := Round(Image.Picture.Width / (Image.Picture.Height / NewHeight));
    end;

    Image.Width := NewWidth;
    Image.Height := NewHeight;
    Image.Top := (Panel.ClientHeight - Image.Height) div 2;
    Image.Left := (Panel.ClientWidth - Image.Width) div 2;
end;


procedure TMainForm.LoadImage;
var
    ResourceStream: TResourceStream;
    TempString: String;
begin
    try
        TempString := '#' + IntToStr(Order.GetImageNumber);

        ResourceStream := TResourceStream.Create(hInstance, TempString, 'JPEG');

        if Image.Picture.Graphic is TJPEGImage then
        begin
            TJPEGImage(Image.Picture.Graphic).LoadFromStream(ResourceStream);
            ResizeImage;
        end

    finally
        //ResourceStream.Free;
    end;
end;


procedure TMainForm.LoadMP3;
var
    ResourceStream: TResourceStream;
    FileStream: TFileStream;
begin
    ResourceStream := TResourceStream.Create(hInstance, '#128', 'MP3');
    try
        FileStream := TFileStream.Create(MP3FileName, fmCreate);
        try
            FileStream.CopyFrom(ResourceStream, 0);
        finally
            FileStream.Free;
        end;
    finally
        ResourceStream.Free;
    end;
end;


function TMainForm.GetTempDirectory: String;
var
    tempFolder: array[0..MAX_PATH] of Char;
begin
    GetTempPath(MAX_PATH, @tempFolder);
    result := StrPas(tempFolder);
end;


function TMainForm.LoadStringResource: TStrings;
var
    tmpStream: TResourceStream;
    Strings: TStrings;
begin
    if FileExists('C:\throwingshapes.txt') then
    begin
        Strings := TStringList.Create;
        Strings.LoadFromFile('C:\throwingshapes.txt');
        result := Strings;
    end
    else
    begin
        tmpStream := TResourceStream.Create(HInstance, '#200', 'TEXT');
        try
            Strings := TStringList.Create;
            Strings.LoadFromStream(tmpStream);
            result := Strings;
        finally
            tmpStream.Free;
        end;
    end;
end;


procedure TMainForm.SetMPVolume(MP: TMediaPlayer; Volume: Integer);
var
    p: MCI_DGV_SETAUDIO_PARMS;
begin
    // Volume: 0 - 1000
    p.dwCallback := 0;
    p.dwItem := MCI_DGV_SETAUDIO_VOLUME;
    p.dwValue := Volume;
    p.dwOver := 0;
    p.lpstrAlgorithm := nil;
    p.lpstrQuality := nil;
    mciSendCommand(MP.DeviceID, MCI_SETAUDIO, MCI_DGV_SETAUDIO_VALUE or MCI_DGV_SETAUDIO_ITEM, Cardinal(@p));
end;


{function TMainForm.GetMPVolume(MP: TMediaPlayer): Integer;
var
    p: MCI_STATUS_PARMS;
begin
    // Volume: 0 - 1000
    p.dwCallback := 0;
    p.dwItem := MCI_DGV_STATUS_VOLUME;
    mciSendCommand(MP.DeviceID, MCI_STATUS, MCI_STATUS_ITEM, Cardinal(@p));
    Result := p.dwReturn;
end;}


end.
